@@ -12,13 +12,13 @@
 
 """A shim around PyCA for DSA public and private keys"""
 
-from ...asn1 import der_encode, der_decode
-
 from cryptography.exceptions import InvalidSignature
 from cryptography.hazmat.backends import default_backend
 from cryptography.hazmat.primitives.hashes import SHA1
 from cryptography.hazmat.primitives.asymmetric import dsa
 
+from ...asn1 import der_encode, der_decode
+
 # Short variable names are used here, matching names in the spec
 # pylint: disable=invalid-name
 
@@ -26,16 +26,10 @@ from cryptography.hazmat.primitives.asymmetric import dsa
 class _DSAKey:
     """Base class for shim around PyCA for DSA keys"""
 
-    def __init__(self, p, q, g, y, x=None):
-        self._params = dsa.DSAParameterNumbers(p, q, g)
-        self._pub = dsa.DSAPublicNumbers(y, self._params)
-
-        if x:
-            self._priv = dsa.DSAPrivateNumbers(x, self._pub)
-            self._priv_key = self._priv.private_key(default_backend())
-        else:
-            self._priv = None
-            self._pub_key = self._pub.public_key(default_backend())
+    def __init__(self, params, pub, priv=None):
+        self._params = params
+        self._pub = pub
+        self._priv = priv
 
     @property
     def p(self):
@@ -71,25 +65,60 @@ class _DSAKey:
 class DSAPrivateKey(_DSAKey):
     """A shim around PyCA for DSA private keys"""
 
+    def __init__(self, params, pub, priv, priv_key):
+        super().__init__(params, pub, priv)
+        self._priv_key = priv_key
+
+    @classmethod
+    def construct(cls, p, q, g, y, x):
+        """Construct a DSA private key"""
+
+        params = dsa.DSAParameterNumbers(p, q, g)
+        pub = dsa.DSAPublicNumbers(y, params)
+        priv = dsa.DSAPrivateNumbers(x, pub)
+        priv_key = priv.private_key(default_backend())
+
+        return cls(params, pub, priv, priv_key)
+
+    @classmethod
+    def generate(cls, key_size):
+        """Generate a new DSA private key"""
+
+        priv_key = dsa.generate_private_key(key_size, default_backend())
+        priv = priv_key.private_numbers()
+        pub = priv.public_numbers
+        params = pub.parameter_numbers
+
+        return cls(params, pub, priv, priv_key)
+
     def sign(self, data):
         """Sign a block of data"""
 
-        signer = self._priv_key.signer(SHA1())
-        signer.update(data)
-        return der_decode(signer.finalize())
+        return der_decode(self._priv_key.sign(data, SHA1()))
 
 
 class DSAPublicKey(_DSAKey):
     """A shim around PyCA for DSA public keys"""
 
+    def __init__(self, params, pub, pub_key):
+        super().__init__(params, pub)
+        self._pub_key = pub_key
+
+    @classmethod
+    def construct(cls, p, q, g, y):
+        """Construct a DSA public key"""
+
+        params = dsa.DSAParameterNumbers(p, q, g)
+        pub = dsa.DSAPublicNumbers(y, params)
+        pub_key = pub.public_key(default_backend())
+
+        return cls(params, pub, pub_key)
+
     def verify(self, data, sig):
         """Verify the signature on a block of data"""
 
-        verifier = self._pub_key.verifier(der_encode(sig), SHA1())
-        verifier.update(data)
-
         try:
-            verifier.verify()
+            self._pub_key.verify(der_encode(sig), data, SHA1())
             return True
         except InvalidSignature:
             return False
