@@ -24,8 +24,11 @@ class _Ed25519Key(SSHKey):
     """Handler for Ed25519 public key encryption"""
 
     algorithm = b'ssh-ed25519'
+    sig_algorithms = (b'ssh-ed25519',)
 
     def __init__(self, vk, sk):
+        super().__init__()
+
         self._vk = vk
         self._sk = sk
 
@@ -33,12 +36,20 @@ class _Ed25519Key(SSHKey):
         # This isn't protected access - both objects are _Ed25519Key instances
         # pylint: disable=protected-access
 
-        return (isinstance(other, self.__class__) and
+        return (isinstance(other, type(self)) and
                 self._vk == other._vk and self._sk == other._sk)
 
     def __hash__(self):
         return hash(self._vk)
 
+    @classmethod
+    def generate(cls, algorithm):
+        """Generate a new Ed25519 private key"""
+
+        # pylint: disable=unused-argument
+
+        return cls(*libnacl.crypto_sign_keypair())
+
     @classmethod
     def make_private(cls, vk, sk):
         """Construct an Ed25519 private key"""
@@ -81,14 +92,24 @@ class _Ed25519Key(SSHKey):
 
         return String(self._vk)
 
-    def sign(self, data):
+    def encode_agent_cert_private(self):
+        """Encode Ed25519 certificate private key data for agent"""
+
+        return self.encode_ssh_private()
+
+    def sign(self, data, algorithm):
         """Return a signature of the specified data using this key"""
 
+        # pylint: disable=unused-argument
+
         if self._sk is None:
             raise ValueError('Private key needed for signing')
 
+        if algorithm not in self.sig_algorithms:
+            raise ValueError('Unrecognized signature algorithm')
+
         sig = libnacl.crypto_sign(data, self._sk)
-        return b''.join((String(self.algorithm), String(sig[:-len(data)])))
+        return b''.join((String(algorithm), String(sig[:-len(data)])))
 
     def verify(self, data, sig):
         """Verify a signature of the specified data using this key"""
@@ -96,7 +117,7 @@ class _Ed25519Key(SSHKey):
         try:
             packet = SSHPacket(sig)
 
-            if packet.get_string() != self.algorithm:
+            if packet.get_string() not in self.sig_algorithms:
                 return False
 
             sig = packet.get_string()
@@ -108,11 +129,13 @@ class _Ed25519Key(SSHKey):
 
 
 try:
+    # pylint: disable=wrong-import-position,wrong-import-order
     import libnacl
 except (ImportError, OSError): # pragma: no cover
     pass
 else:
     register_public_key_alg(b'ssh-ed25519', _Ed25519Key)
 
-    register_certificate_alg(b'ssh-ed25519-cert-v01@openssh.com',
+    register_certificate_alg(1, b'ssh-ed25519',
+                             b'ssh-ed25519-cert-v01@openssh.com',
                              _Ed25519Key, SSHCertificateV01)
