@@ -15,16 +15,22 @@
 import asyncio
 import functools
 import ipaddress
+import os
 import platform
 import socket
 
+from collections import OrderedDict
 from random import SystemRandom
 
+import asyncssh
+
 from .constants import DEFAULT_LANG
 
 
-# Provide a global to test if we're on Python 3.5 or later
+# Provide globals to test if we're on various Python versions
+python344 = platform.python_version_tuple() >= ('3', '4', '4')
 python35 = platform.python_version_tuple() >= ('3', '5', '0')
+python352 = platform.python_version_tuple() >= ('3', '5', '2')
 
 
 # Define a version of randrange which is based on SystemRandom(), so that
@@ -33,12 +39,37 @@ _random = SystemRandom()
 randrange = _random.randrange
 
 
+# Avoid deprecation warning for asyncio.async()
+if python344:
+    ensure_future = asyncio.ensure_future
+else: # pragma: no cover
+    ensure_future = asyncio.async    # pylint: disable=deprecated-method
+
+
 def all_ints(seq):
     """Return if a sequence contains all integers"""
 
     return all(isinstance(i, int) for i in seq)
 
 
+# Default file names in .ssh directory to read private keys from
+_DEFAULT_KEY_FILES = ('id_ed25519', 'id_ecdsa', 'id_rsa', 'id_dsa')
+
+def load_default_keypairs(passphrase=None):
+    """Return a list of default keys from the user's home directory"""
+
+    result = []
+
+    for file in _DEFAULT_KEY_FILES:
+        try:
+            file = os.path.join(os.path.expanduser('~'), '.ssh', file)
+            result.extend(asyncssh.load_keypairs(file, passphrase))
+        except OSError:
+            pass
+
+    return result
+
+
 # Punctuation to map when creating handler names
 _HANDLER_PUNCTUATION = (('@', '_at_'), ('.', '_dot_'), ('-', '_'))
 
@@ -88,6 +119,12 @@ def ip_network(addr):
     return ipaddress.ip_network(_normalize_scoped_ip(addr) + mask)
 
 
+if python352:
+    async_iterator = lambda iter: iter
+else:
+    async_iterator = asyncio.coroutine
+
+
 def async_context_manager(coro):
     """Decorator for methods returning asynchronous context managers
 
@@ -139,6 +176,27 @@ def async_context_manager(coro):
         return coro
 
 
+class Record:
+    """General-purpose record type with fixed set of fields"""
+
+    __slots__ = OrderedDict()
+
+    def __init__(self, *args, **kwargs):
+        for k, v in self.__slots__.items():
+            setattr(self, k, v)
+
+        for k, v in zip(self.__slots__, args):
+            setattr(self, k, v)
+
+        for k, v in kwargs.items():
+            setattr(self, k, v)
+
+    def __repr__(self):
+        return '%s(%s)' % (type(self).__name__,
+                           ', '.join('%s=%r' % (k, getattr(self, k))
+                                     for k in self.__slots__))
+
+
 class Error(Exception):
     """General SSH error"""
 
