@@ -21,22 +21,22 @@
 
 import asyncio, asyncssh, sys
 
-async def handle_connection(stdin, stdout, stderr):
+async def handle_session(stdin, stdout, stderr):
     env = stdout.channel.get_environment()
     if env:
         keywidth = max(map(len, env.keys()))+1
-        stdout.write('Environment:\r\n')
+        stdout.write('Environment:\n')
         for key, value in env.items():
-            stdout.write('  %-*s %s\r\n' % (keywidth, key+':', value))
+            stdout.write('  %-*s %s\n' % (keywidth, key+':', value))
         stdout.channel.exit(0)
     else:
-        stderr.write('No environment sent.\r\n')
+        stderr.write('No environment sent.\n')
         stdout.channel.exit(1)
 
 async def start_server():
     await asyncssh.listen('', 8022, server_host_keys=['ssh_host_key'],
                           authorized_client_keys='ssh_user_ca',
-                          session_factory=handle_connection)
+                          session_factory=handle_session)
 
 loop = asyncio.get_event_loop()
 
